
;;;;;;;; UART using SCC/SIO	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	IF	TNC2		; do if TNC2

VINTPP	; init the parallel port and default them
	RET

VINTSP	; init the serial port (chan B,	SIO)
	LD	HL,VINTS10		; pt to	table
	LD	B,8			; number of bytes to load
	LD	C,SIOBC			; where	to load	them
	OTIR				; do the load
	RET				; that's all
VINTS10	DB	4,046H			; X16, 1 stop, even, disable parity
	DB	1,004H			; status affects vector
	DB	3,0C1H			; 8 bits rx, !CTS, rx enable
	DB	5,0EAH			; DTR on, 8 bits rx, tx	on, RST	on

VGET	; console input
	IN	A,(SIOBC)		; get status
	AND	1			; remove crud
	JR	Z,VGET			; wait till char ready
	IN	A,(SIOBD)		; get data
	AND	7FH			; only 7 bits
	RET

VGETE	; console input	with echo
	CALL	VGET			; get the char
	CALL	VPUT			; put the char
	RET

VPUT	; console output
	PUSH	AF			; save on stack
VPUT10	IN	A,(SIOBC)		; get status
	AND	4			; remove crud
	JR	Z,VPUT10		; wait till tx ready
	POP	AF			; get char from	stack
	OUT	(SIOBD),A		; put it
	RET

VRBRDY	; if char waiting return CY == 1
	IN	A,(SIOBC)		; get status
	AND	1			; test for input
	RET	Z			; return if none
	SCF				; some,	CY <- 1
	RET

VTBRDY	; ret CY = 1 if	transmit buffer	is ready for data
	IN	A,(SIOBC)		; get status
	AND	4			; ready
	RET	Z			; ret CY = 0 if	not
	SCF				; ready, CY <- 1
	RET

VCTSRDY	; see if CTS, CY=1 if so
	LD	A,10H			; get SIO reset command
	OUT	(SIOBC),A		; output to port B control
	IN	A,(SIOBC)		; get port B control
	AND	20H			; mask for CTS bit
	RET	Z			; CY is zero, return if not CTS
	SCF				; mark CTS
	RET				; and return

	ENDIF
	IF	BIOUART

;;;;;;;	UART using bit I/O (8255) ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;	is it really worth saving $10 on a uart	to do serial bit I/O???

VGETE	; get console input with echo using bit	I/O
	PUSH	HL			; save all regs
	PUSH	DE
	PUSH	BC
	LD	HL,BRDLY-2		; get delay value
	LD	B,8			; number of data bits
VGETE10	CALL	VGETSB			; get a	sample
	JR	C,VGETE10		; (VGETE10) loop until spacing
	PUSH	HL			; save delay value
	LD	HL,BRDLY-2/2		; get 1/2 of it
	CALL	KILTIME			; and wait that	amount
	POP	HL			; restore old delay value
	CALL	VGETSB			; get a	sample in middle of bit
	JR	C,VGETE10		; loop if was noise
	CALL	VPUTSB			; else echo start bit
VGETE20	CALL	KILTIME			; wait a full bit time
	CALL	VGETSB			; get next bit
	CALL	VPUTSB			; echo it
	RR	C			; rotate data into C
	DJNZ	VGETE20			; loop until all bits rxed
	LD	HL,BRDLY/2		; get 1/2 of it
	CALL	KILTIME			; delay	over rest of last bit
	SCF				; get mark
	CALL	VPUTSB			; echo mark
	LD	A,C			; restore char
	AND	07FH			; only 7 lsb are wanted
	POP	BC			; restore all regs
	POP	DE
	POP	HL
	RET

VGET	; console input	without	echo
	PUSH	HL			; save all regs
	PUSH	DE
	PUSH	BC
	LD	HL,BRDLY		; get delay value
	LD	B,8			; number of data bits
VGET10	CALL	VGETSB			; get a	sample
	JR	C,VGET10		; loop until spacing
	PUSH	HL			; save delay value
	LD	HL,BRDLY/2		; get 1/2 of it
	CALL	KILTIME			; and wait that	amount
	POP	HL			; restore old delay value
	CALL	VGETSB			; get a	sample in middle of bit
	JR	C,VGET10		; loop if was noise
VGET20	CALL	KILTIME			; wait a full bit time
	CALL	VGETSB			; get next bit
	RR	C			; rotate data into C
	DJNZ	VGET20			; loop until all bits rxed
	LD	HL,BRDLY/2		; get 1/2 of it
	CALL	KILTIME			; delay	over rest of last bit
	LD	A,C			; restore char
	AND	07FH			; only 7 lsb are wanted
	POP	BC			; restore all regs
	POP	DE
	POP	HL
	RET

VPUT	; console output using bit I/O
	PUSH	HL			; save all regs
	PUSH	DE
	PUSH	BC
	LD	C,A			; save the char	in C
	LD	HL,BRDLY		; get delay value
	CALL	KILTIME			; and wait that	amount
	CALL	KILTIME			; and wait that	amount
	LD	B,11			; number of data bits +	stop bits
	XOR	A			; clear	A and CY
	CALL	VPUTSB			; output start bit
	CALL	KILTIME			; wait a full bit time
VPUT20	SCF				; fill with stop bits
	RR	C			; rotate next bit into C
	CALL	VPUTSB			; output bit
	CALL	KILTIME			; delay	to complete last bit
	DJNZ	VPUT20			; loop until all + stop sent
	POP	BC			; restore all regs
	POP	DE
	POP	HL
	RET

;;;;;;;;;;
; 'vintsp'  init port that functions as bit io
;;;;;;;;;;
VINTSP
	RET				; no port setup required

;;;;;;;;;;
; 'vgetsb'  get a serial bit input from input, invert, and put in CY
;;;;;;;;;;
VGETSB	IN	A,(IPORT0)		; get C	port input
	AND	01H			; mask bit
	SCF				; assume 0 input
	RET	Z			; ret if so
	CCF				; else ~CY
	RET

;;;;;;;;;;
; 'vputsb'  put CY to C7
;;;;;;;;;;
VPUTSB	JR	C,VPUTS10
	LD	A,0E1H			; space
	JR	VPUTS20
VPUTS10	LD	A,0E0H			; mark
VPUTS20	OUT	(OPORT0),A		; output the bit
	RET

;;;;;;;;;;
; 'vrbrdy' return CY=1 if activity
;;;;;;;;;;
VRBRDY	CALL	VGETSB			; get input sample, CY=1 if mark
	CCF				; CY=1 if space
	RET

;;;;;;;;;;
; 'vtbrdy' fake	out transmitter	ready
;;;;;;;;;;
VTBRDY	SCF				; mark ready
	RET

;;;;;;;;;;
; 'vctsrdy' fake out CTS ready
;;;;;;;;;;
VCTSRDY	SCF				; mark ready
	RET

	ENDIF
